using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace QD_GUI
{
    public partial class OptionsWindow : Form
    {
        public OptionsWindow()
        {
            InitializeComponent();
        }

        private void OptionsWindow_Load(object sender, EventArgs e)
        {
            txtQDPath.Text = Properties.Settings.Default.qd_FileName;
            txtQDInfoPath.Text = Properties.Settings.Default.qdinfo_FileName;
            txtQDUrl.Text = Properties.Settings.Default.qd_Url;
            txtQDInfoUrl.Text = Properties.Settings.Default.qdinfo_Url;

            lblQDLastModifiedValue.Text = Properties.Settings.Default.qd_LastModified;
            lblQDLastUpdateValue.Text = Properties.Settings.Default.qd_LastUpdate;

            lblQDInfoLastModifiedValue.Text = Properties.Settings.Default.qdinfo_LastModified;
            lblQDInfoLastUpdateValue.Text = Properties.Settings.Default.qdinfo_LastUpdate;

            checkboxOnlyParseQueue.Checked = Properties.Settings.Default.OnlyParseQueue;

            switch (Properties.Settings.Default.SystemType)
            {
                case "windows":
                    radioSystemTypeWindows.Checked = true;
                    break;
                case "linux":
                    radioSystemTypeLinux.Checked = true;
                    break;
                case "mac":
                    radioSystemTypeMacX86.Checked = true;
                    break;
                default:
                    radioSystemTypeAuto.Checked = true;
                    break;
            }

            if (Properties.Settings.Default.ForceUTC == true)
            {
                // check the respective checkbox
                radioButtonForceUTC.Checked = true;

                // uncheck the other checkboxes
                radioButtonUseDefaultTimeZone.Checked = false;
                radioButtonOverrideTimeZone.Checked = false;
            }
            else if (!String.IsNullOrEmpty(Properties.Settings.Default.OverrideTimeZone))
            {
                // check the respective checkbox
                radioButtonOverrideTimeZone.Checked = true;
                txtOverrideTimeZone.Text = Properties.Settings.Default.OverrideTimeZone;
                
                // uncheck the other checkboxes
                radioButtonForceUTC.Checked = false;
                radioButtonUseDefaultTimeZone.Checked = false;
            }
            else
            {
                // check the respective checkbox
                radioButtonUseDefaultTimeZone.Checked = true;

                // uncheck the other checkboxed
                radioButtonOverrideTimeZone.Checked = false;
                radioButtonForceUTC.Checked = false;
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.qd_FileName = txtQDPath.Text;
            Properties.Settings.Default.qdinfo_FileName = txtQDInfoPath.Text;
            Properties.Settings.Default.qd_Url = txtQDUrl.Text;
            Properties.Settings.Default.qdinfo_Url = txtQDInfoUrl.Text;

            // Options

            // -p,  Don't look in other files than queue.dat
            Properties.Settings.Default.OnlyParseQueue = checkboxOnlyParseQueue.Checked;
            
            // -s {linux, windows, mac-ppc, mac-x86}  System type of queue.dat
            if(radioSystemTypeWindows.Checked == true) 
            {
                Properties.Settings.Default.SystemType = "windows";
            }
            else if(radioSystemTypeLinux.Checked == true)
            {
                Properties.Settings.Default.SystemType = "linux";
            }
            //else if (radioSystemTypeMac.Checked == true)
            //{
            //    Properties.Settings.Default.SystemType = "mac";
            //}
            else if (radioSystemTypeMacPPC.Checked == true)
            {
                Properties.Settings.Default.SystemType = "mac-ppc";
            }
            else if (radioSystemTypeMacX86.Checked == true)
            {
                Properties.Settings.Default.SystemType = "mac-x86";
            }
            else if (radioSystemTypeAuto.Checked == true)
            {
                Properties.Settings.Default.SystemType = "auto";
            }
            else
            {
                Properties.Settings.Default.SystemType = "auto";
            }
            
            // -t, -z  Override time zone
            if(radioButtonUseDefaultTimeZone.Checked == true)
            {
                // Unset the other options         
                Properties.Settings.Default.OverrideTimeZone = "";
                Properties.Settings.Default.ForceUTC = false;
            }
            else if (radioButtonOverrideTimeZone.Checked == true)
            {
                // use: qd.exe -t +HHMM
                Properties.Settings.Default.OverrideTimeZone = txtOverrideTimeZone.Text;

                // Unset the other options         
                Properties.Settings.Default.ForceUTC = false;
            }
            else if (radioButtonForceUTC.Checked == true)
            {
                // use: qd.exe -z
                Properties.Settings.Default.ForceUTC = radioButtonForceUTC.Checked;

                // Unset the other options
                Properties.Settings.Default.OverrideTimeZone = "";
            }
            else
            {
                // Unset the other options         
                Properties.Settings.Default.OverrideTimeZone = "";
                Properties.Settings.Default.ForceUTC = false;
            }

            Properties.Settings.Default.Save();

            Close();
        }

        private void btnQDPath_Click(object sender, EventArgs e)
        {
            ofdQD.Title = "Select a File";
            ofdQD.InitialDirectory = @"C:\";
            ofdQD.Filter = "All Files|*.*";
            ofdQD.CheckFileExists = false;
            ofdQD.FileName = "qd.exe";

            if (!String.IsNullOrEmpty(txtQDPath.Text))
            {
                int iLastIndex = txtQDPath.Text.LastIndexOf("\\");
                int iLength = txtQDPath.Text.Length - (txtQDPath.Text.Length - iLastIndex);

                string sDirectory = txtQDPath.Text.Substring(0, iLength);
                string sFilename = txtQDPath.Text.Substring(iLastIndex + 1);

                ofdQD.InitialDirectory = sDirectory;
                ofdQD.FileName = sFilename;
            }

            if (ofdQD.ShowDialog() != DialogResult.Cancel)
            {
                txtQDPath.Text = ofdQD.FileName;
            }
        }

        private void btnQDInfoPath_Click(object sender, EventArgs e)
        {
            ofdQDInfo.Title = "Select a File";
            ofdQDInfo.InitialDirectory = @"C:\";
            ofdQDInfo.Filter = "All Files|*.*";
            ofdQDInfo.CheckFileExists = false;
            ofdQDInfo.FileName = "qdinfo.dat";

            if (!String.IsNullOrEmpty(txtQDInfoPath.Text))
            {
                int iLastIndex = txtQDInfoPath.Text.LastIndexOf("\\");
                int iLength = txtQDInfoPath.Text.Length - (txtQDInfoPath.Text.Length - iLastIndex);

                string sDirectory = txtQDInfoPath.Text.Substring(0, iLength);
                string sFilename = txtQDInfoPath.Text.Substring(iLastIndex + 1);

                ofdQDInfo.InitialDirectory = sDirectory;
                ofdQDInfo.FileName = sFilename;
            }

            if (ofdQDInfo.ShowDialog() != DialogResult.Cancel)
            {
                txtQDInfoPath.Text = ofdQDInfo.FileName;
            }
        }

        private void radioButtonOverrideTimeZone_CheckedChanged(object sender, EventArgs e)
        {
            txtOverrideTimeZone.Enabled = radioButtonOverrideTimeZone.Checked;
        }

        private void txtOverrideTimeZone_Leave(object sender, EventArgs e)
        {
            if(!String.IsNullOrEmpty(txtOverrideTimeZone.Text)) 
            {
                string sRegex = @"^(\+|-)\d{4}$";
                if (!Regex.IsMatch(txtOverrideTimeZone.Text, sRegex))
                {
                    StringBuilder oString = new StringBuilder();
                    oString.Append("Error: Invalid time zone override format!\r\n");

                    MessageBox.Show(oString.ToString());

                    txtOverrideTimeZone.Focus();
                }
            }
        }
    }
}